.react-dropdown-tree-select .tag {
    color: #575757;
    background-color: #f4f4f4;
    border-radius: 2px;
    border: 1px solid #e9e9e9;
    display: inline-block;
    padding: 0 0 0 .5em;
    font-size: .875rem;
    margin: 0.25em;
}

.react-dropdown-tree-select .tag-remove {
    background-color: transparent;
    border: 0;
    color: #a0a0a0;
    cursor: pointer;
    font-size: 1rem;
    font-weight: bold;
    line-height: 100%;
    margin: 0;
    outline: none;
    padding: 0 .5em 0;
}

.react-dropdown-tree-select .node>label {
    cursor: pointer;
    display: inline-block;
    line-height: 1em;
    height: 1em;
}

.react-dropdown-tree-select .tag-list {
    display: inline;
    padding: 0;
    margin: 0;
}

.react-dropdown-tree-select .tag-item {
    display: inline-block;
}

.react-dropdown-tree-select .tag-item .search {
    border: none;
    box-shadow: none;
    background: none;
    outline: none;
    margin: 0;
}

.react-dropdown-tree-select .tag-item:last-child {
    margin-right: 4px;
}

.react-dropdown-tree-select .node {
    list-style: none;
    white-space: nowrap;
    padding: 4px;
}

.react-dropdown-tree-select .node.leaf.collapsed {
    display: none;
}

.react-dropdown-tree-select .node.disabled>* {
    color: gray;
    cursor: not-allowed;
}

.react-dropdown-tree-select .node.match-in-children.hide .node-label {
    opacity: .5;
}

.react-dropdown-tree-select .toggle {
    white-space: pre;
    font-style: normal;
    font-weight: bold;
    display: inline-block;
    width: 1.5em;
    line-height: 1em;
    height: 1em;
}

.react-dropdown-tree-select .toggle:after {
    content: " ";
}

.react-dropdown-tree-select .toggle.collapsed:after {
    cursor: pointer;
    content: "+";
}

.react-dropdown-tree-select .toggle.expanded:after {
    cursor: pointer;
    content: "-";
}

.react-dropdown-tree-select .searchModeOn .toggle {
    display: none;
}

.react-dropdown-tree-select .checkbox-item {
    vertical-align: middle;
    margin: 2px 0.5em 0 0 !important;
    height: 1em;
}

.react-dropdown-tree-select .checkbox-item.simple-select {
    display: none;
}

.react-dropdown-tree-select .hide:not(.match-in-children) {
    display: none;
}

.react-dropdown-tree-select .dropdown {
    position: relative;
    display: table;
    width: 100%;
}

.react-dropdown-tree-select .dropdown .dropdown-trigger {
    padding: 0;
    line-height: 20px;
    max-height: 200px;
    display: inline-block;
    overflow: auto;
    border: 1px solid #ccc;
    width: 100%;
    box-shadow: inset 0 1px 2px rgba(0,0,0,0.1);
}

.react-dropdown-tree-select .dropdown .dropdown-trigger.arrow {
    cursor: pointer;
}

/* .react-dropdown-tree-select .dropdown .dropdown-trigger.arrow.bottom:after {
    content: "\25BC";
    vertical-align: middle;
    color: #3c3c3c;
    margin-right: 2px;
} */

/* .react-dropdown-tree-select .dropdown .dropdown-trigger.arrow.top:after {
    content: "\25B2";
    vertical-align: middle;
    color: #3c3c3c;
    margin-right: 2px;
} */

.react-dropdown-tree-select .dropdown .dropdown-trigger.arrow.disabled {
    cursor: not-allowed;
}

.react-dropdown-tree-select .dropdown .dropdown-trigger.arrow.disabled.bottom:after {
    color: #b9b9b9;
}

.react-dropdown-tree-select .dropdown .dropdown-content {
    position: absolute;
    padding: 4px;
    z-index: 1;
    background: #fff;
    border-top: 1px solid rgba(0, 0, 0, .05);
    -webkit-box-shadow: 0 5px 8px rgba(0, 0, 0, .15);
    box-shadow: 0 5px 8px rgba(0, 0, 0, .15);
}

.react-dropdown-tree-select .dropdown .dropdown-content ul {
    margin: 0;
    padding: 0;
}
