footer {
    color: $white;
    font-size: .75rem;
    margin-top: 3.5rem;
    padding: 2.5rem 0 3.5rem;

    .links {
        background-color: $primary-color;
        padding: .5em 1.2rem;
    }

    a {
        color: $white;
    }
}

@media print {
    footer {
        display: none;
    }
}
